function [gammaMat zetaProposalMat rhoMat sigcVec lambdazVec rVec acceptgam acceptRhoBetween acceptRhoWithin] = GP_MetUniReg(diffsq,diffsqs,J,Jsmall,m,n,y,ar,br,az,bz,ac,bc,zeta,arho,brho,npar,burnin)
% Conducts and returns posterior sampled values for gamma and Theta
% INPUT:
%    diffsq, diffsqs : Covariate data in squared difference form,
%                (X1(i,:)-X2(j,:)).^2, to build GP covariance matrix
%                 to construct likelihood.  Posteriors are proportional to
%                 likelihood x prior.  'diffsq' contains only n(n-1)/2
%                 unique values.  'diffsqs' is of length m(m-1)/2 from
%                 smaller set of m randomly selected points used for
%                 faster inverstion
%   J, Jsmall: Indices for diffsq and diffsqs to reinflate from unique
%              to full set of values 
%   m, n    : see above.
%   y       : Response (nsample,1)
%   ar,br,az,bz,ac,bc: (shape,rate) hyperparameters on gamma priors
%                       for precision parameters
%  zeta:   mean of bernoulli prior on gamma_k
%  (arho,brho): hyperparameters for beta prior on rho_k
%  npar, burnin: Number of sampling iterations and number of burnin
%
% OUTPUT:
%   rhoMat   : (nkeep x p) posterior samples of rho_{1},...,rho_{p}
%   gammaMat  : (nkeep x p) posterior samples of gamma_{1},...,gamma_{p}
%   zetaProposalMat:  (nkeep x p) set of adaptive proposal means for the
%               Bernoulli prior on gamma_{k}
%   sigcVec, lambdazVec, rVec: (nkeep x 1) vector of sampled values for
%                precision parameters.
%   acceptGam, acceptRhoBetween, acceptRhoWithin: acceptance rates for
%           sampling gamma_k, between model sampling of rho_k and 
%           within model sampling of rho_k.
%

% Start values to kick off Gibbs samples with Metropolis moves
L = size(diffsq,2);
% Start parameter values to initialize the MCMC
% lastgamma = ones(1,L);
% lastrho = rand([1,L]);
% load rhoStart
% lastrho = rhoStart;
% lastgamma = zeros([1,L]);
% lastgamma(lastrho<1) = 1;
lastgamma = zeros([1,L]);
lastrho = ones([1,L]);

lastsigc =(ac/bc);
lastr = (ar/br);
lastlambdaz = (az/bz);

% INITIALIZE STRUCTURES for returned values from MCMC
% Model and Covariance Parameters
runs = npar-burnin;
rhoMat =zeros(runs,L);
gammaMat =zeros(runs,L);
gammaTemp = zeros(npar,L);
zetaProposalMat =zeros(runs,L);

% Nuisance parameters
sigcVec = zeros(1,runs);
lambdazVec = zeros(1,runs);
rVec = zeros(1,runs);

% Measuring the acceptance rates for rho and nu in our Metropolis steps
% when gamma_k = 1
proposeRhoBetween = 0;
proposeRhoWithin = 0;
countRhoBetween = 0;
countRhoWithin = 0;
countgamma = 0;
% 'D' is tuning parameter for number of times to re-sample rho_{k}
% for 'within' step when gamma_{k} = 1, meaning x_{k} is in the current
% model.
D = 1;
% 'w' is a tuning parameter to generate multiple candidates for rho_{k}
% from which one is randomly selected to improve mixing.
w = 20;

% Tuning parameters for updating, zeta (p x 1), the mean for the bernoulli
% proposal for gamma_{k}|gamma_{-k}, ...
lambda = .01;
zetaProposal = 0.5*ones([1,L]);
% jitter = 0; 
updateCount = 1;
lastUpdateNum = 1;
updateInterval = 20;
firstUpdate = updateInterval;
stepIncrement = 0.3;

for i = 1:npar
    
    % Initialize of joint posterior density for (gamma,rho) and compute
    % reduced-sized, m x m covariance matrix, CsOld, from randomly chosen
    % m < n data points.  When paired with m x n COld, the full n x n
    % GP covariance matrix, Cproject, may be built and inversion may
    % be accomplished at the lower m x m dimension for faster inversion.
    last = [lastrho lastsigc lastr lastlambdaz lastgamma];
    [fGamRholast CsOld COld] = fpbeta(last,1,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],[],[],[],[],zeta,arho,brho);
     
    for j = 1:L
    % Between model step to jointly sample (gamma_k, rho_k)
    % in a Metropolis step using a Gibbs-type sequential scan.
        candgamma = lastgamma;
        candrho = lastrho;
        component = bernoulli(lambda);
        candgamma(j) = (1-component)*bernoulli(zetaProposal(j)) + component*bernoulli(0.5);
        if(candgamma(j)==1)
            proposeRhoBetween = proposeRhoBetween + 1;
            candrho(j) = rand(1);
            cand = [candrho lastsigc lastr lastlambdaz candgamma];
            [fGamRhocand CsNew CNew] = fpbeta(cand,5,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],[],[],[],[],zeta,arho,brho,CsOld,COld,[],j,lastrho(j));
            alpha = fGamRhocand  - fGamRholast;
            if (log(rand(1)) < alpha)
                lastgamma(j) = candgamma(j);
                lastrho(j) = candrho(j);
                fGamRholast = fGamRhocand; 
                CsOld = CsNew; COld = CNew;
                countgamma = countgamma + 1;
                countRhoBetween = countRhoBetween + 1;
            end;
        elseif(lastgamma(j) ~= 0) % candgamma(j)==0
            candrho(j) = 1;
            cand = [candrho lastsigc lastr lastlambdaz candgamma];
            [fGamRhocand CsNew CNew] = fpbeta(cand,5,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],[],[],[],[],zeta,arho,brho,CsOld,COld,[],j,lastrho(j));
            alpha = fGamRhocand  - fGamRholast;
            if (log(rand(1)) < alpha)
                lastgamma(j) = candgamma(j);
                lastrho(j) = candrho(j);
                fGamRholast = fGamRhocand; 
                CsOld = CsNew; COld = CNew;
                countgamma = countgamma + 1;
            end; 
        end;
        % Within-model move resampling of rho_{k}|gamma_{k} = 1

        if(lastgamma(j)==1)
            for d = 1:D
            proposeRhoWithin = proposeRhoWithin + 1;
            rhoProposal = rand([1,w]);
            candrho(j) = randsample(rhoProposal,1);
%             candrho(j) = rand(1);
            cand = [candrho lastsigc lastr lastlambdaz lastgamma];
            [fGamRhocand CsNew CNew]= fpbeta(cand,5,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],[],[],[],[],zeta,arho,brho,CsOld,COld,[],j,lastrho(j));
            alpha = fGamRhocand - fGamRholast;
            if (log(rand(1)) < alpha)
                    lastrho(j) = candrho(j);
                    fGamRholast = fGamRhocand; 
                    CsOld = CsNew; COld = CNew;
                    countRhoWithin = countRhoWithin + 1;
            end;
            end;
        end; 
    end; %end loop j on rho vector of length L
    gammaTemp(i,:) = lastgamma;
    
    %updating sigc from full conditional 
    last = [lastrho lastsigc lastr lastlambdaz lastgamma]; 
    fsigclast = fpbeta(last,4,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],[],[],ac,bc,[]);
    tune = 1;
    a = tune;
    b = lastsigc/tune;
    candsigc = gamrnd(a,b);
    cand = [lastrho candsigc lastr lastlambdaz lastgamma]; 
    fsigccand = fpbeta(cand,4,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],[],[],ac,bc,[]);
    qlastnew = (tune-1)*log(candsigc) - tune*candsigc/(lastsigc);
    qnewlast = (tune-1)*log(lastsigc) - tune*lastsigc/(candsigc);
    alpha = (fsigccand+qnewlast-fsigclast-qlastnew); 
    if (log(rand(1)) < alpha)
        lastsigc = candsigc;
    end;
    
     %updating lambdaZ from full conditional  
    last = [lastrho lastsigc lastr lastlambdaz lastgamma]; 
    [flambdazlast Cs C Cproject] = fpbeta(last,3,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],az,bz,[],[],[]);
    tune = 1;
    a = tune;
    b = lastlambdaz/tune;
    candlambdaz = gamrnd(a,b);
    cand = [lastrho lastsigc lastr candlambdaz lastgamma]; 
    [flambdazcand Csc Cc Cprojectc] = fpbeta(cand,3,diffsq,diffsqs,J,Jsmall,m,n,y,[],[],az,bz,[],[],[]);
    qlastnew = (tune-1)*log(candlambdaz) - tune*candlambdaz/(lastlambdaz);
    qnewlast = (tune-1)*log(lastlambdaz) - tune*lastlambdaz/(candlambdaz);
    alpha = (flambdazcand+qnewlast-flambdazlast-qlastnew); 
    if (log(rand(1)) < alpha) 
        lastlambdaz = candlambdaz;
        Cs = Csc; C = Cc; Cproject = Cprojectc;
    end;
    
    %updating r from full conditional 
    last = [lastrho lastsigc lastr lastlambdaz lastgamma]; 
    frlast = fpbeta(last,2,diffsq,diffsqs,J,Jsmall,m,n,y,ar,br,[],[],[],[],[],[],[],Cs,C,Cproject);
    tune = 1;
    a = tune;
    b = lastr/tune;
    candr = gamrnd(a,b);
    cand = [lastrho lastsigc candr lastlambdaz lastgamma]; 
    frcand = fpbeta(cand,2,diffsq,diffsqs,J,Jsmall,m,n,y,ar,br,[],[],[],[],[],[],[],Cs,C,Cproject);
    qlastnew = (tune-1)*log(candr) - tune*candr/(lastr);
    qnewlast = (tune-1)*log(lastr) - tune*lastr/(candr);
    alpha = (frcand+qnewlast-frlast-qlastnew); 
    if (log(rand(1)) < alpha)
        lastr = candr;
    end;
    
    %updating 1 x p zeta from full conditionals 
    testInd = (updateCount==1);
    updateNum = updateInterval*updateCount;
    if(i == updateNum*(1-testInd) + firstUpdate*testInd)
        updateCount = updateCount + 1;
        sumGamma = sum(gammaTemp(lastUpdateNum:i,:));
        iterations = i - lastUpdateNum + 1;
        ergAvgZeta = sumGamma/(iterations);
        step = stepIncrement/updateCount;
        zetaProposal = zetaProposal + step*(ergAvgZeta - zetaProposal);
        lastUpdateNum = updateNum;
%         zetaProposal(zetaProposal>=(1-jitter)) = 1 - jitter;
%         zetaProposal(zetaProposal<=jitter) = jitter;
%         zetaProposal(zetaProposal<=jitter) = 0;
    end;
    
    if(i > burnin)
        iterKeep = i - burnin;
        rhoMat(iterKeep,:) = lastrho;
        gammaMat(iterKeep,:) = lastgamma;
        zetaProposalMat(iterKeep,:) = zetaProposal;
        sigcVec(iterKeep) = lastsigc;
        lambdazVec(iterKeep) = lastlambdaz;
        rVec(iterKeep) = lastr;      
    end;
   
end;

%Counting percent of gamma, rho, nu proposals accepted in Metropolis
%moves
    proposals = npar*L;
    acceptRhoBetween = countRhoBetween/proposeRhoBetween;
    acceptRhoWithin = countRhoWithin/proposeRhoWithin;
    acceptgam = countgamma/proposals;