function [normMSPE rootMSPE MSPE ypred yitepred] = GP_PredUniReg(Xtr,Xte,ytr,myi,syi,yite,boundary,gamprob,rho,sigc,lambdaz,r)
% Function performs posterior prediction of response values (ypred) associated
% to a test data design matrix (Xte) based on numerically marginalizing the conditional
% predictive distribution over the posterior distributions for Theta computed during
% MCMC sampling.  The mean square prediction error (MSPE) is also computed
% and returned.
% Input:
%   Xtr, ytr: data used to train the model (perform posterior sampling)
%   myi, syi:  mean and standard deviation of yitr, the unnormalized
%              training data response
%  yite: unnormalized test data response
%  boundary: threshold for P(gamma_k = 1|Y) for inclusion of covariates
%  gamprob: marginal posterior probability of covariate inclusion,
%  P(gamma_k = 1|Y)
%  rho, sigc, lambdaz, r: posterior sample draws from Theta and r
% Output:
%  normMSPE:  MSPE normalized by variance of yite
%  rootMSPE:  sqrt(MSPE)
%  MSPE
%  ypred: standardized prediction of test data responses
%  yitepred: unstandardized prediction of test data responses
% Savitsky/Vannucci (2008 - 2011)

%Define covariates to keep based on decision make via EFDR on rho_k, nu_k
keep = find(gamprob > boundary);
rhokeep = rho(:,keep);

%Update design matrix to select out retained covariates.  Used to peform
%calculation on latent response, Ytest, for test covariates.
Xtrain = Xtr(:,keep);
Xtest = Xte(:,keep);
[difftesq Ite numte numtr] = difference(Xtest,Xtrain);
[difftrsq Itr] = difference(Xtrain,Xtrain);

%Using the test dataset to make predictions. We use the mean of the
%predictive distribution, E(Ytest|Yhat, rho, lambdaz) for each iteration 
%of rho, lambdaz as a function of Yhat, the average of all Y's over 
%the MCMC runs.  We then marginalize the predictive distribution to
% E(Ytest|Yhat) by summing over every 10th value of the MCMC iterations for
% rho and lambdaz.

%Calculating ypred based on marginalizing over MCMC values of rho, nu, sigc
%lambdaz and lambdae.
Jit = .01;
rate = .2;
runs = size(rho,1);
samples = floor(runs*rate);
Dsum = zeros(numte,1);
% Csum = zeros(numte,numte);
for l = 1:samples
    point = floor((1/rate))*l;
    rhosample = rhokeep(point,:);
    sigcsample = sigc(point); sigcinv = 1/sigcsample;
    lambdazsample = lambdaz(point); lambdazinv = 1/lambdazsample;
    rsample = r(point);
    Ltetr = Rusym1TMat(difftesq,Ite,numte,numtr,rhosample,sigcinv,lambdazinv);
    Ltrtr = (1/rsample+Jit)*eye(numtr)+Rusym1TMat(difftrsq,Itr,numtr,numtr,rhosample,sigcinv,lambdazinv);
    Gtrtr = chol(Ltrtr,'lower');
    Utr = Gtrtr\Ltetr';
    vtr = Gtrtr\ytr;    
    D = Utr'*vtr;
    Dsum = Dsum+D; %size D, Dsum = test x 1
end;
ypred = Dsum./samples; %size = test x 1

%Calculating Mean Squared Prediction Error
%normMSPE = mean((yte-ypred).^2);
yitepred = syi*ypred+myi;
MSPE = mean((yite-yitepred).^2);
rootMSPE = sqrt(MSPE);
normMSPE = MSPE/var(yite);