function [L Cs acceptgam acceptRhoBetween acceptRhoWithin MSPE rootMSPE normMSPE numte te ypred yite yitepred Xte Xtr Xtrs yitr ytr syi myi indx boundary rhoMat sigc r lambdaz gammaMat zetaProposalMat rhoprob gamprob EFDRrho EFDRgam eps] = UniRegressionGibbsAdaptBetaPriorBurnin(Xi,yi,numte,psample,npar,burnin,boundary,zeta)
% Inputs a MULTIVARIATE response, y (1 x q), and a design matrix, X, and performs
% Bayesian Variable Selection on a Gaussian Process regression model.  The
% GP covariance function includes 2 sets of parameters for L covariates:
% rho (L x 1) = non-linear, nu (L x 1) = linear.  We also include a set
% of variable selection parameters, gamma (L x 1).  We sample the posterior
% distributions for these parameters, as well as the precision parameters,
% lambdae (the error term in y = Z(X) + e, Z(X) ~ N(0,R)) and lambdaz, the
% scale multiplier or weight for the non-linear covariance term.  We apply
% the FDR thresholding method to gamma to select covariates and to rho and
% nu to select non-linear and linear covariates, respectively.
% INPUT:
%   Xi       : Design matrix (of covariates)
%   yi       : n x 1 vector response
%   numte    : number of observations to set aside for out-of-sample
%             validation
%   psample : percent of training sample to be randomly chosen for lower
%             dimension project on inverse computation for C.  
%             Lower values produce faster computation and more stable C, 
%             but with progresively reduced accuracy. Recommend 0.2 - 0.35.
%   npar    : Number of iterations for a single MCMC run
%   burnin  : Number of the npar to discard
%   boundary: Posterior cut-off boundary for inclusion in model.
% OUTPUT:
%   Sampled Parameters:
%   rhoMat   : (nkeep x p) posterior samples of rho_{1},...,rho_{p}
%   gammaMat  : (nkeep x p) posterior samples of gamma_{1},...,gamma_{p}
%   zetaProposalMat:  (nkeep x p) set of adaptive proposal means for the
%               Bernoulli prior on gamma_{k}
%   gamprob   : Marginal posterior, P(gamma){k} == 1|Y) 
%   rhoprob   : Marginal posterior probability that rho_k < 1.
%   EFDRgam     : Expected FDR associated with posterior threshold.
%   EFDRrho     : Expected FDR associated with posterior threshold.
%   boundary  :  posterior threshold used for EFDR computations
%   sigc      : Samples from marg posterior of affine intercept term,
%             :  lambda_{a} from GP covariance matrix of Savitsky et. al.
%             : (2011)
%   r         : Samples of marginal posterior for error precision term
%   lambdaz   : Samples of marginal posterior for precision multiplier of
%               the exponential covariance term
%   MCMC Performance:
%   MSPE, rootMSPE, normMSPE:  Mean squared prediction error, square root
%                    of MSPE and MSPE normalized by variance of test set
%   acceptgam  : Acceptance rate of Metropolis proposals on gamma_{k}
%   acceptRhoBetween:  Acceptance rate on proposals for rho_{k} on between
%                      model moves
%   acceptRhoWithin: Acceptance rate on proposals for rho_{k} on within
%                     model moves
%   Data:
%   Xtr, ytr, yitr : Training design matrix (normalized to lie in [0,1])
%                    and standardized training reponse and unstandardized 
%                    training response, respectively
%   Xte, yte, yite, te:  Same as above, but for hold out training set used
%                        for prediction.  'te' is the index of (Xi,yi) 
%                        selected for the test set
%   Xtrs, indx     : Index and design matrix of subset of observations
%                   randomly selected for lower order m < numtr projection 
%                    to facilitate inverse computation of C
%   Savitsky and Vannucci (2008 - 2010)

% Capturing runtime
tic;

% Set calculation precision
eps = 1e-32;

% Set fixed hyperparameter values (ae,be,az,bz,zetascalar)

ar = 2.5; %shape hyperparameter for error precision (r)
br = .025; %scale hyperparameter for error precision (r) 
az = 0.5; %shape hyperparameter for exponential weight (lambdaz)
bz = 0.5; %scale hyperparameter for expential weight (lambdaz) 
ac = 0.1; %shape hyperparameter for covariance intercep (sigc)
bc = 0.1; %scale hyperparameter for covariance intercept (sigc)
%zeta = .25; %hyperparameter for p(gamma_k ==1)
arho = 1.0; % hyperparmeter for beta prior on rho_{k}
brho = 1.0; % hyperparmeter for beta prior on rho_{k}
L = size(Xi,2); % Number of predictors

% TRANSFORM {Xi,yi}
%   Normalize yi and transform Xi to [0,1]

[te Xtr Xte ytr yitr yite myi syi] = transform(2,Xi,yi,numte);

% Randomly select sub-set of points to use for "m < n" dimension covariance
% matrix. Returns (m x L) design matrix where L = number of predictors and
% 'indx', the index set of sampled training cases.

[Xtrs indx] = project(Xtr,psample);

% Compute matrix of differenced rows of Xtrs and Xtr for non-symmetric covariance
% matrix and same for Xtrs for symmetric covariance matrix

[diffsq J m n] = difference(Xtrs,Xtr);
[diffsqs Jsmall] = difference(Xtrs,Xtrs);

% M-H runs where we allow for multiple repetitions (reps) of MCMC runs of
% length (npar). Generally, we will execute a single MCMC run and use the
% expected false discovery rate to threshold our covariance parameters and
% covariates based on a cutoff value for the marginal posteriors.

% MCMC

%M-H iterations

[gammaMat zetaProposalMat rhoMat sigc lambdaz r acceptgam acceptRhoBetween acceptRhoWithin Cs] = metGibbsAdaptBetaPriorPartialMultiBurnIn(diffsq,diffsqs,J,Jsmall,m,n,ytr,ar,br,az,bz,ac,bc,zeta,arho,brho,npar,burnin);
        

% MARGINAL POSTERIORS
% Defining alternative threshold method (to the RDVS) based on FDR (Newton,
% 2004).  Note that we no longer use the inert variable to make decisions 
% about which covariates are active.  Instead, we use a cutoff on
% posteriors for rho and gamma. based on an expected FDR target.
% (Sort of like a Bayesian Type I error).

% Compute posterior probability that (parm < 1) based on counts
% Compute EFDR based on probability of null hypothesis 

[gamprob EFDRgam] = margp(2,gammaMat,boundary);
[rhoprob EFDRrho] = margp(1,rhoMat,boundary);


% PREDICTION
%Using the test dataset to make predictions. We use the mean of the
%predictive distribution, E(Ytest|Yhat, rho, lambdaz) for each iteration 
%of rho, lambdaz as a function of Yhat, the average of all Y's over 
%the MCMC runs.  We then marginalize the predictive distribution to
% E(Ytest|Yhat) by summing over every 10th value of the MCMC iterations for
% rho and lambdaz.

[normMSPE rootMSPE MSPE ypred yitepred] = pred(Xtr,Xte,ytr,myi,syi,yite,boundary,gamprob,rhoMat,sigc,lambdaz,r);
% memo: myi, syi are the mean and sd of the training response (before
% standardization). we "re-inflate" the standardized test prediction with
% myi and syi.

save('BH08102','L','Cs','indx','boundary','MSPE','rootMSPE','normMSPE','numte','te','ypred','yitepred','yite','yitr','Xte','Xtr','Xtrs','ytr','syi','myi','acceptgam','acceptRhoBetween','acceptRhoWithin','rhoMat','sigc','r','lambdaz','gammaMat','zetaProposalMat','rhoprob','gamprob','EFDRrho','EFDRgam');

toc
