function [p EFDR] = margp(type,parms,boundary)
% Function computes marginal posterior distributions of variable inclusion
% in two equivalent ways: P(gamma_k = 1 | Y), using gamma 
% and P(rho_k < 1|Y) using rho.  The expected false discovery rate (EFDR)
% is also computed for an input of the threshold value, boundary
% Input:
%  type: 1 = compute P(gamma_k = 1|Y); 2 = compute P(rho_k < 1|Y)
%  parms: either gamma or rho
%  boundary:  threshold on posterior for covariate inclusion
% Output:
%  p: marginal posterior probability of covariate inclusion
%  EFDR: expected false discovery rate
% Savitsky/Vannucci (2008-2011)

L = size(parms,2);
p = zeros(L,1);

switch type
    case 1 %Parameter space, e.g. rho
        for k = 1:L
            counts = parms(:,k);
            p(k) = length(counts(counts<1))/length(counts);
        end;
        
    case 2 %Model space, e.g. gamma
        for k = 1:L
            counts = parms(:,k);
            p(k) = length(counts(counts==1))/length(counts);
        end;
end;
        
%Make decisions in ({0},{1}) based on user input cut-off/boundary.
d = zeros(L,1);
d(p > boundary)=1;

%Compute resultant EFDRrho - Sort of like a Bayesian type I error.
%pnull = 1 - p is probability of null hypothesis.
J = ones(L,1);
pnull = J - p; %Posterior probability of null hypothesis
EFDR = sum(d'*pnull)/sum(d);
